
const path = require('path');

const fs = require('fs');

function trim(s) {
    return (s || '').replace(/^\s*(\S*(\s+\S+)*)\s*$/,'$1');
}

const defOpt = {
    timeout: 10
};

let globOpt = {};

const now = () => parseInt((new Date()).getTime() / 1000, 10);

const cache = {

};

const regex = /\/api\/json\/(.+)/;

const setup = opt => {

    if ( ! opt.dir ) {

        throw `jsonFilesCacheMiddleware: No directory specified`;
    }

    if ( ! fs.existsSync(opt.dir) ) {

        throw `jsonFilesCacheMiddleware: Directory ${opt.dir} doesn't exist`;
    }

    globOpt = Object.assign(defOpt, opt);
};

const get = param => {

    const
        n       = now(),
        keys    = Object.keys(cache)
    ;

    for ( let t, k, i = 0, l = keys.length ; i < l ; i += 1 ) {

        k = keys[i];

        if (k === param) {

            t = cache[k];

            if ( (n - t.created) <= globOpt.timeout ) {

                return t.content;
            }

            return null;
        }
    }

    return null;
};

const set = (path, content) => (cache[path] = {
    content,
    created: now()
});

/**
 * First call setup();
 */
const getDataFromJsonFile = param => {

    let content     = null;

    let error       = null;

    let status      = null;

    content = get(param);

    if ( content === null ) {

        let file = path.resolve(globOpt.dir, `${param}.json`);

        content = '';

        if (fs.existsSync(file)) {

            content = fs.readFileSync(file).toString();

            set(param, content);

            status = `jsonFilesCacheMiddleware: cache just refreshed`;
        }
        else {

            file = path.resolve(globOpt.dir, `${param}.js`);

            if (fs.existsSync(file)) {

                content = eval('require')(path.resolve(globOpt.dir, `${param}`));

                content = JSON.stringify(content);

                set(param, content);

                status = `jsonFilesCacheMiddleware: cache just refreshed`;
            }
            else {

                error = `jsonFilesCacheMiddleware: file: ${file} not found`;
            }
        }
    }
    else {

        status = `jsonFilesCacheMiddleware: from cache`;
    }

    return {
        content,
        error,
        status
    };
};

/**
 * express middleware
 */
const tool = opt => {

    setup(opt);

    return (req, res) => {

        const param     = trim(req.params[0]);

        let tmp = {
            error: null,
            status: null,
            content: null
        };

        if (param) {

            tmp = getDataFromJsonFile(param);
        }

        (tmp.error !== null) && (res.set('X-api-error', tmp.error));

        (tmp.status !== null) && (res.set('X-api-status', tmp.status));

        if (tmp.content === null) {

            res
                .status(404)
                .end()
            ;
        }
        else {

            res
                .set('Content-type', 'application/json; charset=utf-8')
                .send(tmp.content)
            ;
        }
    };
};


tool.regex      = regex;

tool.getDataFromJsonFile    = getDataFromJsonFile;

tool.setup      = setup;

module.exports = tool;