
import React, { Component, Fragment } from 'react';

import {
    Image,
    Button,
    Input
} from 'semantic-ui-react';

import Tooltip from 'rc-tooltip';

import './ImageWithSizesTooltip.scss';

import PropTypes from 'prop-types';

const change = (flag, url) => {

    const parts = (url || "").split("?")[0].match(/^(.*)(\.[^\.]+)$/);

    if (parts && parts.length === 3) {

        return `${parts[1]}[${flag}].jpg`;
    }

    return url;
};

export default class ImageWithSizesTooltip extends Component {
    static propTypes: {
        media: PropTypes.object.isRequired,
    };
    constructor(...args) {
        super(...args);

        this.state = {
            loading: false,
            list: []
        };
    }
    contentEditable() {
    }
    onVisibleChange = () => setTimeout(() => {
        this.refs.forEach(e => e.setAttribute('contenteditable', true));
    })
    render() {

        this.refs = [];

        const origin = (function () {
            try {
                return location.origin;
            }
            catch (e) {
                return false;
            }
        }());

        const {
            img,
            media,
            ...props
        } = this.props;

        const list = Object.keys(media).map(flag => ({
            key: flag,
            url: change(flag, img.url),
            ...media[flag]
        }));
        return (
            <div className="image-with-sizes-thumb" {...props} >
                <Tooltip
                    overlayClassName="image-with-sizes-tooltip"
                    onVisibleChange={this.onVisibleChange}
                    overlay={
                        <div>
                            <h4>All allowed sizes:</h4>
                            <div className="grid">
                                {list.map(item => (
                                    <div key={item.url}>
                                        <div>
                                            <b>{item.width} x {item.height}</b>
                                        </div>
                                        <div
                                            onClick={() => document.execCommand('selectAll',false,null)}
                                        >
                                            <table>
                                                <tbody>
                                                <tr>
                                                    <td>
                                                        <b>local:</b>
                                                    </td>
                                                    <td>
                                                        <div ref={div => div && this.refs.push(div)}>{item.url}</div>
                                                    </td>
                                                </tr>
                                                <tr>
                                                    <td>
                                                        <b>public:</b>
                                                    </td>
                                                    <td>
                                                        <div ref={div => div && this.refs.push(div)}>{origin}{item.url}</div>
                                                    </td>
                                                </tr>
                                                </tbody>
                                            </table>
                                        </div>
                                        <div>
                                            <a href={item.url} target="_blank">link</a>
                                        </div>
                                    </div>
                                ))}
                            </div>
                        </div>
                    }
                    trigger="click"
                    align={{
                        offset: [20, 0]
                    }}
                    placement="rightBottom"
                >
                    <Button
                        loading={this.state.loading}
                        disabled={this.state.loading}
                        circular
                        icon="unordered list"
                        onClick={e => e.preventDefault()}
                    />
                </Tooltip>
                <img
                    key={img.id}
                    id={img.id}
                    src={img.url}
                    size='small'
                    {...props}
                />
            </div>
        );
    }
}