
import React, { Component } from 'react';

import PropTypes from 'prop-types';

const getLoc = () => {

    let loc;

    try {

        loc = location.href;
    }
    catch (e) {

        log(`Can't access location.href`);
    }

    return loc;
}

/**
 * Inspiration: https://stackoverflow.com/a/39094299
 */
export default class Onbeforeunload extends Component {
    constructor(...args) {
        super(...args)

        this.state = {
            on: false
        }
    }
    static propTypes = {
        when: PropTypes.bool.isRequired,
        unloadMessage: PropTypes.string.isRequired
    };
    onUnload(msg) { // the method that will be used for both add and remove event
        // https://developer.mozilla.org/en-US/docs/Web/Events/beforeunload
        // https://stackoverflow.com/a/10311375
        let loc = getLoc();

        return e => {

            let curloc = getLoc();

            if (curloc && curloc === loc) {

                e = e || window.event;

                // For IE and Firefox prior to version 4
                if (e) {

                    e.returnValue = msg;
                }

                // For Safari
                return msg;
            }

            log(`current location '${curloc}' is different then at the moment of mounting onbeforeunload '${loc}'`)
        }
    }
    on = () => {
        // log('on', this.props, this.state, 'this.props.when && ! this.state.on', this.props.when && ! this.state.on)
        if ( this.props.when && ! this.state.on ) {

            try {
                window.onbeforeunload = this.onUnload(this.props.unloadMessage);
            }
            catch (e) {
                log('Onbeforeunload error: ', e);
            }

            this.setState({on:true});
        }
    }
    off = () => {
        // log('off', this.props, this.state, '! this.props.when && this.state.on', ! this.props.when && this.state.on)
        if ( ! this.props.when && this.state.on ) {

            try {
                window.onbeforeunload = undefined;
            }
            catch (e) {
                log('Onbeforeunload error: ', e);
            }

            this.setState({on:false});
        }
    }
    trigger = () => {
        this.on();
        this.off();
    }
    componentWillUnmount() {
        this.off();
    }
    render() {

        setTimeout(() => this.trigger());

        return null;
    }
}