
import React, { Component, Fragment } from 'react';

import {
    Prompt as RouterPrompt
} from 'react-router';

import Onbeforeunload from './Onbeforeunload';

/**
 * Use like here:
 * https://reacttraining.com/react-router/web/example/preventing-transitions g(react router 4 prevent transition)
 * but it is necessary to provide one extra parameter
 *   unloadMessage
 *
    <Prompt
        when={this.state.blocked}
        message={location =>
            `Form contains unsaved data, do you really want to discard changes without saving by navigating to: ${location.pathname} ?`
        }
        unloadMessage="Form contains unsaved data..."
    />

    it is also possible to give just unloadMessage message if you don't want to use ${location.pathname} in message,
    then unloadMessage parameter will be used as a message for react router default Prompt component
 */
export default class Prompt extends Component {
    render() {

        let {
            unloadMessage,
            message
        } = this.props;

        if ( ! message && unloadMessage ) {

            message = unloadMessage;
        }

        return (
            <Fragment>
                <RouterPrompt
                    {...this.props}
                    message={message}
                />
                <Onbeforeunload {...this.props} />
            </Fragment>
        );
    }
}