
import React, { Component } from 'react';

import PropTypes from 'prop-types';

const node = typeof global !== 'undefined' && Object.prototype.toString.call(global.process) === '[object process]';

function unique(pattern) { // node.js require('crypto').randomBytes(16).toString('hex');
    pattern || (pattern = 'xyxyxy');
    return pattern.replace(/[xy]/g,
        function(c) {
            var r = Math.random() * 16 | 0,
                v = c == 'x' ? r : (r & 0x3 | 0x8);
            return v.toString(16);
        });
}

const next = [];

export const refresh = () => {

    while (next.length > 9) {

        next.shift();
    }

    let tmp;

    do {

        tmp = unique();

    } while (next.indexOf(tmp) > -1);

    next.push(tmp);

    return tmp;
};

const load = (function () {

    let promise;

    return () => {

        if ( ! promise ) {

            promise = new Promise(resolve => {

                if ( ! node ) {

                    if (window.grecaptcha) {

                        log('ReCaptcha.js: exist')

                        resolve();
                    }
                    else {

                        log('ReCaptcha.js: creating')
                        // https://developers.google.com/recaptcha/docs/display#explicit_render
                        // g(Explicitly render the reCAPTCHA widget)

                        const name = 'grecaptcha_ready';

                        window[name]    = resolve;
                        const script    = document.createElement('script');
                        const first     = document.getElementsByTagName('script')[0];
                        script.async    = 1;
                        script.src      = `https://www.google.com/recaptcha/api.js?onload=${name}&render=explicit`;

                        first.parentNode.insertBefore(script, first);
                    }
                }

            });
        }

        return promise;
    }
}());

export default class ReCaptcha extends Component {
    static defaultProps = {
        opt         : {}
    }
    static propTypes = {
        name        : PropTypes.string,
        opt         : PropTypes.object,
        sitekey     : PropTypes.string.isRequired,
        onChange    : PropTypes.func,
        refresh     : PropTypes.string,
    };

    constructor(props) {

        super(props);

        this.state = {
            id: false,
        }

        this.executeReady = true;
    }

    shouldComponentUpdate = (nextProps, nextState) => {

        // log('shouldComponentUpdate', nextProps);

        if (nextProps.refresh) {

            if (this.refresh !== nextProps.refresh) {

                this.refresh = nextProps.refresh;

                // log('trigger refresh', ( ! node ) , window.grecaptcha , Number.isInteger(this.state.id));

                if ( ( ! node ) && window.grecaptcha && Number.isInteger(this.state.id)) {

                    log('trigger for sure');

                    grecaptcha.reset(
                        this.state.id,
                    );
                }
            }
        }

        return false;
    }
    componentWillUnmount() {

        this.executeReady = false;

        log('ReCaptcha.js: componentWillUnmount')
    }
    componentDidMount() {

        this.refresh = this.props.refresh;

        load().then(() => this.ready());
    }
    ready = () => {

        if ( ! this.executeReady ) {

            return log('ReCaptcha.js: not executeReady');
        }

        if ( ! this.ref ) {

            return log('ReCaptcha.js: no reference');
        }

        log('ReCaptcha.js: ready', this.ref);

        this.id = (function () {

            let id = '';

            do {

                id = unique();

            } while (document.getElementById(id));

            return id;
        }());

        // log('id', this.id);

        this.ref.setAttribute('id', this.id);

        const {
            opt = {},
            onChange = () => {},
            sitekey,
        } = this.props;

        const id = grecaptcha.render(
            this.ref,
            {...opt, ...{
                sitekey,
                callback: onChange
            }},
        );

        if (Number.isInteger(id)) {

            this.setState({id});
        }

        // log('back', id);
    }
    render () {
        return (
            <div ref={el => this.ref = el} />
        );
    }
}


/**

 import React, { Component, Fragment } from 'react';

 import ReCaptcha, { refresh } from 'roderic/libs/react/ReCaptcha';

 import { Link } from "react-router-dom";

 export default class ReCaptchaTest extends Component {
    constructor(...args) {

        super(...args);

        this.state = {
            text: '',
            solution: 'def',
            refresh: 'init',
            inc: 0,
        };
    }
    onSubmit = e => {

        // e.preventDefault();

    }
    refreshRecaptcha = () => {
        this.setState({
            refresh: refresh()
        });
    }
    inc = () => this.setState(ps => ({
        inc: ps.inc + 1
    }));
    render() {

        const {
            text,
            solution,
            refresh,
            inc,
        } = this.state;

        return (
            <Fragment>
                <form onSubmit={this.onSubmit} method="post" action="/">
                    <label>
                        text
                        <br/>
                        <input
                            type="text"
                            value={text}
                            onChange={e => this.setState({
                                text: e.target.value
                            })}
                            name="text"
                        />
                    </label>
                    <br/>
                    <ReCaptcha
                        sitekey="6LdZk.............WVOmgrYRqeV"
                        onChange={solution => this.setState({solution})}
                        refresh={refresh}
                        inc={inc}
                        opt={{
                            hl: 'fr' // https://developers.google.com/recaptcha/docs/display#js_param
                        }}
                    />
                    <fieldset style={{
                        border: '1px solid gray',
                        margin: '2px',
                        padding: '12px',
                    }}>
                        <legend style={{
                            margin: '2px',
                            padding: '12px',
                            width: 'auto',
                        }}>Solution:</legend>
                        {solution}
                    </fieldset>

                    <br/>
                    <input type="submit" value="submit"/>
                </form>
                <Link to="/tree">back</Link>

                <button onClick={this.refreshRecaptcha}>refresh</button>
                <button onClick={this.inc}>inc</button>
            </Fragment>
        );
    }
}

 */