
const path = require('path');

const launch = require(path.resolve(__dirname, '..', '..', 'driver.js'));

const { getUrlTest, getUrlEnv } = require(path.resolve(__dirname, '..', '..', 'driver.js'));

const config = require('../../config');

describe('006 scss', async () => {

    let browser, page;

    beforeAll(async () => {

        browser = await launch(true);

        page = await browser.page(true);
    });

    afterAll(async () => {
        await browser.close();
    });

    const common = async () => {

        const page = await browser.page(true);

        await page.getServerEnv('/en/test-page');

        // await page.sleepSec(2);

        await page.click('.link-puppeteer');

        // await page.sleepSec(2);

        const text = await page.evaluate((/* one, two, three */) => new Promise(resolve => {

            var inter = setInterval(() => {

                const el = document.querySelector('.puppeteer-test');

                if (el) {

                    clearInterval(inter);

                    resolve({data: el.innerText});
                }

            }, 200);

        })/* , 'one', 'two', { three: 'four' } */);

        await page.sleepSec(0.5);
    }

    it('.puppeteer-scss-test border', async () => {

        await common();

        const border = await page.evaluate((/* one, two, three */) => new Promise(resolve => {

            var inter = setInterval(() => {

                const el = document.querySelector('.puppeteer-scss-test');

                if (el) {

                    clearInterval(inter);

                    resolve({data: window.getComputedStyle(el).getPropertyValue('border')});
                }

            }, 200);

        })/* , 'one', 'two', { three: 'four' } */);

        expect(border.data).toBe("3px dashed rgb(255, 0, 0)");
    });

    it('text color', async () => {

        await common();

        const border = await page.evaluate((/* one, two, three */) => new Promise(resolve => {

            var inter = setInterval(() => {

                const el = document.querySelector('.puppeteer-scss-test .text');

                if (el) {

                    clearInterval(inter);

                    resolve({data: window.getComputedStyle(el).getPropertyValue('background-color')});
                }

            }, 200);

        })/* , 'one', 'two', { three: 'four' } */);

        expect(border.data).toBe("rgb(173, 216, 230)");
    });

    it('png height', async () => {

        await common();

        const border = await page.evaluate((/* one, two, three */) => new Promise(resolve => {

            var inter = setInterval(() => {

                const el = document.querySelector('.puppeteer-scss-test .png');

                if (el) {

                    const height = parseInt(window.getComputedStyle(el).getPropertyValue('height'), 10);

                    if (height > 20) {

                        clearInterval(inter);

                        resolve({data: height});
                    }
                }

            }, 200);

        })/* , 'one', 'two', { three: 'four' } */);

        expect(border.data).toBeGreaterThan(20);
    });

    it('did included scss loaded image?', async () => {

        await common();

        const border = await page.evaluate((/* one, two, three */) => new Promise(resolve => {

            var inter = setInterval(() => {

                const el = document.querySelector('.puppeteer-scss-test .inc');

                if (el) {

                    const height = parseInt(window.getComputedStyle(el).getPropertyValue('height'), 10);

                    if (height > 20) {

                        clearInterval(inter);

                        resolve({data: height});
                    }
                }

            }, 200);

        })/* , 'one', 'two', { three: 'four' } */);

        expect(border.data).toBeGreaterThan(50);
    });
});