
const isObject  = require('lodash/isObject');

const isArray   = require('lodash/isArray');

const node      = require('detect-node');

let envcache = false;

/**
 * Private method to get environment variables depending on detected environment
 * @returns {boolean}
 */
const get = name => {

    let env = envcache;

    if (node) {

        if ( ! envcache) {

            envcache = process.env || {};

            if (envcache['NODE_ENV'] === 'development') {

                envcache.WEBPACK_MODE = 'dev';
            }
            else {

                envcache.WEBPACK_MODE = 'prod';
            }
        }

        env = envcache;
    }
    else if (env === false) {

        throw `roderic/webpack/dotenv.js: to use roderic/webpack/dotenv.js in browser mode first run dotenv.webload(.env) in webpack web entrypoint. key: '${name}'`;
    }

    return env;
}

/**
 * General purpose tool to get any envcacheironement variable
 * @param name
 * @param args you can specify default value if there is no ${name} environment variable
 */
const tool = (name, ...args) => {

    let env = get(name);

    if ( ! name ) { // if no name given then return all, but copy first

        return Object.assign({}, env);
    }

    if (typeof env[name] === 'undefined') {

        if (args.length > 0) {

            return args[0];
        }

        throw `roderic/webpack/dotenv.js: there is no environment variable '${name}'`;
    }

    return env[name];
};

tool.has = name => {

    let env = get(name);

    if (typeof env[name] === 'undefined') {

        return false;
    }

    return true;
}

/**
 * To execute in *.entry.js
 * @param file
 */
tool.webload = params => {

    if ( ! isObject(params) ) {

        throw `roderic/webpack/dotenv.js:webload: params must be an object`
    }

    if ( isArray(params) ) {

        throw `roderic/webpack/dotenv.js:webload: params can't be an array`;
    }

    // console.log('params.ENV_SECURE', params.ENV_SECURE, JSON.stringify(params.ENV_SECURE));

    if ( ! node ) {

        if ( typeof params.ENV_SECURE === 'undefined' ) {

            throw `roderic/webpack/dotenv.js:webload: params should have ENV_SECURE value`;
        }

        if ( params.ENV_SECURE !== 'true' ) {

            throw `roderic/webpack/dotenv.js:webload: ENV_SECURE should be 'true' in web mode`;
        }
    }

    envcache = params;
};

/**
 * To execute in preprocessor.js
 * @param file
 */

tool.preprocess = (dotenvfile, targetjsonfile) => {

    if ( ! node ) {

        throw `roderic/webpack/dotenv.js:preprocess: don't call this method in web mode`;
    }

    console.log("\n");

    const env = tool.parse(dotenvfile);

    env.ENV_ORIGIN      = 'preprocessor';

    env.WEBPACK_MODE    = process.env.WEBPACK_MODE || 'dev';

    console.log("Web exposed environemt variables: \n\n");

    const max = Object.keys(env).reduce((acc, val) => {

        const l = val.length;

        if (l > acc) {

            return l;
        }

        return acc;
    }, 0);

    Object.keys(env).map(key => {

        const l = key.length;

        let k = key;

        if (l < max) {

            k += ' '.repeat(max - l);
        }

        console.log(k, ':', env[key])
    });

    console.log("\n");

    const fs        = eval('require')('fs');

    fs.writeFileSync(targetjsonfile, JSON.stringify(env, null, 4));

    if (fs.existsSync(targetjsonfile)) {

        console.log(`File '${targetjsonfile}' successfully saved...`);
    }
    else {

        throw `Preprocessor: Can't save '${targetjsonfile}'...`;
    }

    console.log("\n");
}

tool.parse = (file, secure = true) => {

    if ( ! node ) {

        throw `roderic/webpack/dotenv.js:parse: don't call this method in web mode`;
    }

    const fs        = eval('require')('fs');

    const dotenv    = eval('require')('dotenv');

    if ( ! fs.existsSync(file) ) {

        throw `File '${file}' doesn't exist`;
    }

    let content = fs.readFileSync(file).toString();

    content = content.split("\n").map(e => {

        if (e.indexOf('export ') === 0) {

            e = e.substring(7);
        }

        return e;

    }).join("\n");

    content = dotenv.parse(content);

    content.ENV_SECURE = secure ? 'true' : 'false';

    return Object.keys(content).reduce((acc, key) => {

        if ( ! secure || key.indexOf('PROTECTED_') !== 0) {

            acc[key] = content[key];
        }

        return acc;

    }, {});
}

(function () {

    if (typeof window !== 'undefined') {

        window.env = tool;
    }

}());

module.exports = tool;

if ( ! node) {

    window.env = tool;
}